using Godot;
using System;

public class ChallengeButton : Button
{
    PackedScene audioStreamPlayer;
    AudioStreamSample asFocus;
    AudioStreamSample asSelect;

    [Export]
    string referencePath = "";
    [Export]
    bool startFocused = false;
    [Export]
    Node playerVariables;

    public override void _Ready()
    {
        audioStreamPlayer = (PackedScene)ResourceLoader.Load("res://Sound/AudioStreamPlayer.tscn");
        asFocus = (AudioStreamSample)ResourceLoader.Load("res://Sound/m_button.wav");
        asSelect = (AudioStreamSample)ResourceLoader.Load("res://Sound/m_select.wav");

        playerVariables = GetNode<Node>("/root/PlayerVariables");

        Connect("mouse_entered", this, "onButtonMouseEntered");
        Connect("focus_exited", this, "onFocused");
        Connect("button_down", this, "onDown");
        Connect("pressed", this, "onButtonPressed");

        if (startFocused)
        {
            GrabFocus();
        }
    }

    public void PlaySound(AudioStreamSample sample)
    {
        AudioStreamPlayer asp = (AudioStreamPlayer)audioStreamPlayer.Instance();
        asp.Stream = sample;
        GetTree().CurrentScene.CallDeferred("add_child", asp);
    }

    public void onButtonMouseEntered()
    {
        GrabFocus();
    }

    public void onFocused()
    {
        PlaySound(asFocus);
    }

    public void onDown()
    {
        PlaySound(asSelect);
    }

    public void onButtonPressed()
    {
        if (referencePath != "")
        {
            playerVariables.Set("Mode", 2);
            GetTree().ChangeScene(referencePath);
        }
        else
        {
            GetTree().Quit();
        }
    }


}
