using Godot;
using System;

public class Crawlbug : KinematicBody2D
{
    // nodes
    Node stats;
    AnimationPlayer animationPlayer;
    AnimationPlayer hitAnimationPlayer;
    RayCast2D leftRayCast;
    RayCast2D rightRayCast;
    KinematicBody2D player;
    Area2D swordHitbox;
    CollisionShape2D hurtboxCollision;
    CollisionShape2D hitboxCollision;
    PackedScene audioStreamPlayer;
    AudioStreamSample asHit;
    // constants
    const int ACCELERATION = 800;
    const int MAX_FALL_SPEED = 200;
    const int KNOCKBACK = 500;
    const int GRAVITY = 400;
    const float FRICTION = 0.3f;
    const int SOUL = 10;
    // vectors
    Vector2 upVector = new Vector2(0, -1);
    Vector2 motion = new Vector2(0, 0);
    // variables
    Area2D hitArea;
    int playerDirection;
    int damage;
    bool dead = false;

    int health = 2;
    int direction = -1;

    //===========================

    enum States
    {
        walk,
        fall,
        death
    }
    States state;


    public override void _Ready()
    {
        animationPlayer = GetNode<AnimationPlayer>("AnimationPlayer");
        hitAnimationPlayer = GetNode<AnimationPlayer>("HitAnimationPlayer");
        leftRayCast = GetNode<RayCast2D>("LeftRayCast");
        rightRayCast = GetNode<RayCast2D>("RightRayCast");

        player = GetParent().GetNode<KinematicBody2D>("../Player");
        stats = player.GetNode<Node>("Stats");
        swordHitbox = GetParent().GetNode<Area2D>("../Player/HitboxPivot/SwordHitbox");
        hurtboxCollision = GetNode<CollisionShape2D>("Hurtbox/CollisionShape2D");
        hitboxCollision = GetNode<CollisionShape2D>("Hitbox/CollisionShape2D");

        audioStreamPlayer = (PackedScene)ResourceLoader.Load("res://Sound/AudioStreamPlayer.tscn");
        asHit = (AudioStreamSample)ResourceLoader.Load("res://Sound/e_hit.wav");

        state = States.fall;
    }

    public override void _PhysicsProcess(float delta)
    {

        if (state == States.walk) // WALK
        {
            // BEHAVIOUR
            if ((!leftRayCast.IsColliding() && direction == -1) || (!rightRayCast.IsColliding() && direction == 1) || IsOnWall())
            {
                direction = -direction;
            }

            motion.x += direction * ACCELERATION * delta;
            Friction(FRICTION);

            Gravity(delta);
            motion = MoveAndSlide(motion, upVector);

            // TRANSITIONS TRIGGERS
            if (!IsOnFloor())
            {
                state = States.fall;
            }

            // ANIMATION
            if (direction == -1) animationPlayer.Play("WalkLeft");
            else if (direction == 1) animationPlayer.Play("WalkRight");
        }
        else if (state == States.fall) // FALL
        {
            // BEHAVIOUR
            motion.x = motion.x * 0.9f;
            Gravity(delta);
            motion = MoveAndSlide(motion, upVector);

            // TRANSITIONS TRIGGERS
            if (IsOnFloor())
            {
                state = States.walk;
            }

            // ANIMATION
            if (direction == -1) animationPlayer.Play("IdleLeft");
            else if (direction == 1) animationPlayer.Play("IdleRight");
        }
        else if (state == States.death) // DEATH
        {
            if (!dead)
            {
                dead = true;
                if ((int)player.Get("currentLevel") != (int)player.Get("LAST_LEVEL"))
                {
                    stats.Set("Souls", (int)stats.Get("Souls") + SOUL);
                }
                stats.Set("Kills", (int)stats.Get("Kills") + 1);
            }

            // BEHAVIOUR
            Friction(FRICTION);
            Gravity(delta);
            motion = MoveAndSlide(motion, upVector);
        }
    }

    public void Gravity(float delta)
    {
        motion.y = Mathf.Min(motion.y + GRAVITY * delta, MAX_FALL_SPEED);
    }

    public void Friction(float friction)
    {
        motion.x = Mathf.Lerp(motion.x, 0, friction);
    }

    public void PlaySound(AudioStreamSample sample)
    {
        AudioStreamPlayer asp = (AudioStreamPlayer)audioStreamPlayer.Instance();
        asp.Stream = sample;
        GetTree().CurrentScene.AddChild(asp);
    }

    public void onHurtboxAreaEntered(Area2D area)
    {
        playerDirection = (int)player.Get("direction");
        damage = (int)swordHitbox.Get("damage");
        if (area.Name == "LavaArea")
        {
            damage = 5;
        }
        health -= damage;
        this.hitAnimationPlayer.Play("Start");
        motion.x = playerDirection * KNOCKBACK;
        PlaySound(asHit);

        if (health > 0)
        {
            direction = playerDirection;
        }
        else if (health <= 0)
        {
            motion.y = -50;
            direction = -playerDirection;
            if (direction == -1) animationPlayer.Play("DeathLeft");
            else if (direction == 1) animationPlayer.Play("DeathRight");

            hurtboxCollision.SetDeferred("disabled", true);
            hitboxCollision.SetDeferred("disabled", true);

            hitArea = area;
            state = States.death;

        }
    }

}
