using Godot;
using System;

public class EndArea : Area2D
{
    Node playerVariables;
    KinematicBody2D player;
    Node stats;
    Camera2D camera;
    Timer sceneChangeTimer;
    PackedScene audioStreamPlayer;
    AudioStreamSample asEnd;

    string scenePath = "res://Menu/EndScreen.tscn";

    [Signal] public delegate void playerEnteredEndSignal();

    public override void _Ready()
    {
        playerVariables = GetNode<Node>("/root/PlayerVariables");
        player = GetTree().CurrentScene.GetNode<KinematicBody2D>("Player");
        stats = player.GetNode<Node>("Stats");
        camera = GetParent().GetNode<Camera2D>("../Player/PlayerCamera");
        sceneChangeTimer = GetNode<Timer>("SceneChangeTimer");
        audioStreamPlayer = (PackedScene)ResourceLoader.Load("res://Sound/AudioStreamPlayer.tscn");
        asEnd = (AudioStreamSample)ResourceLoader.Load("res://Sound/boss_death2.wav");
    }

    public void onEndAreaBodyEntered(KinematicBody2D body)
    {
        this.EmitSignal(nameof(playerEnteredEndSignal));
        playerVariables.Set("Kills", stats.Get("Kills"));
        body.Set("state", 11);
        PlaySound(asEnd);
        camera.Call("FadeIn", 5f);
        sceneChangeTimer.Start(5);
    }

    public void PlaySound(AudioStreamSample sample)
    {
        AudioStreamPlayer asp = (AudioStreamPlayer)audioStreamPlayer.Instance();
        asp.Stream = sample;
        GetTree().CurrentScene.AddChild(asp);
    }

    public void onSceneChangeTimerTimeout()
    {
        GetTree().ChangeScene(scenePath);
    }

}
