using Godot;
using System;

public class EndScreen : Node2D
{
    Node playerVariables;

    AnimationPlayer scrAnimationPlayer;
    Label modeLabel;
    Label berriesLabel;
    Label killsLabel;
    Label escapeTimeLabel;
    Label escapeGradeLabel;
    Button goToMenuButton;
    Timer timer;

    PackedScene audioStreamPlayer;
    AudioStreamSample asVictory;
    AudioStreamSample asLabelShow;
    AudioStreamSample asNotFull;

    int index = 0;
    int BERRY_COUNT = 22;
    int ENEMY_COUNT = 80;
    int modeIndex;
    string mode;
    int berryCollected;
    int enemyKilled;
    string escapeTime;
    string escapeGrade;

    public override void _Ready()
    {
        playerVariables = GetNode<Node>("/root/PlayerVariables");

        scrAnimationPlayer = GetNode<AnimationPlayer>("ScreenAnimationPlayer");
        modeLabel = GetNode<Label>("ModeLabel");
        berriesLabel = GetNode<Label>("BerriesLabel");
        killsLabel = GetNode<Label>("KillsLabel");
        escapeTimeLabel = GetNode<Label>("EscapeTimeLabel");
        escapeGradeLabel = GetNode<Label>("EscapeGradeLabel");
        goToMenuButton = GetNode<Button>("GoBackButton");
        timer = GetNode<Timer>("ShowLabelTimer");

        audioStreamPlayer = (PackedScene)ResourceLoader.Load("res://Sound/AudioStreamPlayer.tscn");
        asVictory = (AudioStreamSample)ResourceLoader.Load("res://Sound/victory.wav");
        asLabelShow = (AudioStreamSample)ResourceLoader.Load("res://Sound/m_select.wav");
        asNotFull = (AudioStreamSample)ResourceLoader.Load("res://Sound/p_swing.wav");

        berryCollected = (int)playerVariables.Get("Berries");
        enemyKilled = (int)playerVariables.Get("Kills");
        modeIndex = (int)playerVariables.Get("Mode");
        escapeTime = (String)playerVariables.Get("escapeTime");
        escapeGrade = (String)playerVariables.Get("escapeGrade");

        if (modeIndex == 0) mode = "Normal";
        else if (modeIndex == 1) mode = "Assist";
        else if (modeIndex == 2) mode = "CHALLENGE";
        else mode = "Speedrun";

        goToMenuButton.Visible = false;
        modeLabel.Visible = false;
        modeLabel.Text = $"Mode: {mode}";
        berriesLabel.Visible = false;
        berriesLabel.Text = "Berries collected: " + berryCollected + "/" + BERRY_COUNT;
        killsLabel.Visible = false;
        killsLabel.Text = "Enemies killed: " + enemyKilled + "/" + ENEMY_COUNT;
        escapeTimeLabel.Visible = false;
        escapeTimeLabel.Text = "Escape time: " + escapeTime;
        escapeGradeLabel.Visible = false;
        escapeGradeLabel.Text = escapeGrade;

        scrAnimationPlayer.Play("FadeOut");

        PlaySound(asVictory);
        playerVariables.Call("DeleteData");
    }


    public void PlaySound(AudioStreamSample sample)
    {
        AudioStreamPlayer asp = (AudioStreamPlayer)audioStreamPlayer.Instance();
        asp.Stream = sample;
        GetTree().CurrentScene.AddChild(asp);
    }

    public void onTimerTimeout()
    {
        if (index == 0)
        {
            index++;
        }
        else if (index == 1)
        {
            if (mode == "Speedrun")
            {
                index = 4;
            }
            else
            {
                index++;
            }
            modeLabel.Visible = true;
            PlaySound(asNotFull);
        }
        else if (index == 2)
        {
            index++;
            berriesLabel.Visible = true;
            if (berryCollected == BERRY_COUNT)
            {
                PlaySound(asLabelShow);
            }
            else
            {
                PlaySound(asNotFull);
            }
        }
        else if (index == 3)
        {
            index++;
            killsLabel.Visible = true;
            if (enemyKilled == ENEMY_COUNT)
            {
                PlaySound(asLabelShow);
            }
            else
            {
                PlaySound(asNotFull);
            }
        }
        else if (index == 4)
        {
            index++;
            escapeTimeLabel.Visible = true;
            PlaySound(asNotFull);
        }
        else if (index == 5)
        {
            index++;
            escapeGradeLabel.Visible = true;
            if (escapeGrade == "A" || escapeGrade == "A+" || escapeGrade == "GODLIKE")
            {
                PlaySound(asLabelShow);
            }
            else
            {
                PlaySound(asNotFull);
            }
        }
        else if (index == 6)
        {
            index++;
            goToMenuButton.Visible = true;
            goToMenuButton.GrabFocus();
            PlaySound(asNotFull);
            timer.Stop();
        }
    }

}
