using Godot;
using System;

public class Enemy : KinematicBody2D
{
    // nodes
    AnimationPlayer animationPlayer;
    AnimationPlayer hitAnimationPlayer;
    RayCast2D leftRayCast;
    RayCast2D rightRayCast;
    KinematicBody2D player;
    Area2D swordHitbox;
    Area2D hurtbox;
    CollisionShape2D hurtboxCollision;
    CollisionShape2D hitboxCollision;
    // constants
    const int ACCELERATION = 0;   // 800
    const int MAX_FALL_SPEED = 0; // 200
    const int GRAVITY = 0;        // 400
    const float FRICTION = 0;     // 0.3f
    // vectors
    Vector2 upVector = new Vector2(0, -1);
    Vector2 motion = new Vector2(0, 0);
    // variables
    int playerDirection;
    int knockback;
    int damage;

    int health = 0;
    int direction = -1;

    //===========================
    
    enum States
    {
        death
    }
    States state;
    

    public override void _Ready()
    {
        animationPlayer = GetNode<AnimationPlayer>("AnimationPlayer");
        hitAnimationPlayer = GetNode<AnimationPlayer>("HitAnimationPlayer");
        leftRayCast = GetNode<RayCast2D>("LeftRayCast");
        rightRayCast = GetNode<RayCast2D>("RightRayCast");

        player = GetParent().GetNode<KinematicBody2D>("../Player");
        swordHitbox = GetParent().GetNode<Area2D>("../Player/HitboxPivot/SwordHitbox");
        hurtbox = GetNode<Area2D>("Hurtbox");
        hurtboxCollision = GetNode<CollisionShape2D>("Hurtbox/CollisionShape2D");
        hitboxCollision = GetNode<CollisionShape2D>("Hitbox/CollisionShape2D");

        //state = States.fall;
    }

    public override void _PhysicsProcess(float delta)
    {
        // State machine
    }

    public void Gravity(float delta)
    {
        motion.y = Mathf.Min(motion.y + GRAVITY * delta, MAX_FALL_SPEED);
    }

    public void Friction(float friction)
    {
        motion.x = Mathf.Lerp(motion.x, 0, friction);
    }

    public void onHurtboxAreaEntered(Area2D area)
    {
        playerDirection = (int)player.Get("direction");
        knockback = (int)swordHitbox.Get("knockback");
        damage = (int)swordHitbox.Get("damage");

        health -= damage;
        motion.x = playerDirection * knockback;
        if (health >= 0)
        {
            hitAnimationPlayer.Play("Start");
            if (health > 0)
            {
                direction = playerDirection;
            }
            else if (health == 0)
            {
                motion.y = -50;
                direction = -playerDirection;
                if (direction == -1) animationPlayer.Play("DeathLeft");
                else if (direction == 1) animationPlayer.Play("DeathRight");

                hurtboxCollision.SetDeferred("disabled", true);
                hitboxCollision.SetDeferred("disabled", true);
                state = States.death;
            }
        }
    }

}
