using Godot;
using System;

public class FallingRocksTriggerArea : Area2D
{
    Node playerVariables;
    KinematicBody2D playerBody;
    Camera2D camera;
    Timer timer;
    PackedScene fallingRock;

    Area2D endArea;

    Random r = new Random();
    bool falling = false;
    bool disabled = false;

    public override void _Ready()
    {
        playerVariables = GetNode<Node>("/root/PlayerVariables");
        playerBody = GetParent().GetNode<KinematicBody2D>("../Player");
        camera = GetTree().CurrentScene.GetNode<Camera2D>("Player/PlayerCamera");
        timer = GetNode<Timer>("Timer");
        fallingRock = (PackedScene)ResourceLoader.Load("res://Hazards/FallingRock/FallingRock.tscn");
        endArea = GetParent().GetNode<Area2D>("EndArea");

        playerBody.Connect("lastLevelResetSignal", this, "onLastLevelResetSignal");
        endArea.Connect("playerEnteredEndSignal", this, "RockFallStop");

        if ((int)playerVariables.Get("Mode") == 3)
        {
            disabled = true;
        }
    }


    public void onFallingRocksTriggerAreaBodyEntered(KinematicBody2D body)
    {
        if (!falling && !disabled)
        {
            falling = true;
            timer.Start(1);
        }
    }

    public void DoubleRockSpawn()
    {
        int x1 = r.Next(48, 208);
        int x2 = r.Next(288, 432);

        RockSpawn(x1);
        RockSpawn(x2);
    }

    public void RockSpawn(int x)
    {
        KinematicBody2D rock = (KinematicBody2D)fallingRock.Instance();
        rock.GlobalPosition = new Vector2(x, camera.GlobalPosition.y - 200);
        rock.Set("initialSpeed", 50);
        rock.Set("z_index", 1);
        GetTree().CurrentScene.CallDeferred("add_child", rock);
    }

    public void onTimerTimeout()
    {
        DoubleRockSpawn();
    }

    public void RockFallStop()
    {
        timer.Stop();
    }

    public void onLastLevelResetSignal()
    {
        falling = false;
        timer.Stop();
    }

}
