using Godot;
using System;

public class GetDash : Area2D
{
    Node playerVariables;
    AnimationPlayer animationPlayer;
    PackedScene floatingLabel;

    PackedScene audioStreamPlayer;
    AudioStreamSample asPickup;
    bool picked = false;

    float time = 0.0f;
    [Export]
    float speed = 5f;
    [Export]
    float power = 0.1f;

    public override void _Ready()
    {
        playerVariables = GetNode<Node>("/root/PlayerVariables");
        animationPlayer = GetNode<AnimationPlayer>("AnimationPlayer");
        floatingLabel = (PackedScene)ResourceLoader.Load("res://UI/FloatingLabel.tscn");

        audioStreamPlayer = (PackedScene)ResourceLoader.Load("res://Sound/AudioStreamPlayer.tscn");
        asPickup = (AudioStreamSample)ResourceLoader.Load("res://Sound/berry_pickup.wav");

        animationPlayer.Play("Idle");
    }

    public override void _PhysicsProcess(float delta)
    {
        time = Mathf.Wrap(time + delta * speed, -Mathf.Pi, Mathf.Pi);
        GlobalPosition = new Vector2(GlobalPosition.x, GlobalPosition.y + (Mathf.Sin(time) * power));
    }

    public void onGetDashBodyEntered(KinematicBody2D body)
    {
        if (!picked)
        {
            picked = true;
            body.Set("canDash", true);
            playerVariables.Set("CanDash", true);
            animationPlayer.Play("Picked");

            Label label = (Label)floatingLabel.Instance();
            label.Text = "DASH\nPress\nC / L";
            label.RectGlobalPosition = this.GlobalPosition - new Vector2(20, 20);
            GetTree().CurrentScene.AddChild(label);

            PlaySound(asPickup);
        }
    }

    public void Delete()
    {
        QueueFree();
    }

    public void PlaySound(AudioStreamSample sample)
    {
        AudioStreamPlayer asp = (AudioStreamPlayer)audioStreamPlayer.Instance();
        asp.Stream = sample;
        GetTree().CurrentScene.AddChild(asp);
    }
}
