using Godot;
using System;
using System.Collections.Generic;
using System.Linq;
using Newtonsoft.Json;

public class PlayerVariables : Node
{
    private const string SAVE_PATH = @"user://save.json";
    private static DataModel _data = new DataModel();

    bool loaded = false;
    bool makeTransition = false;
    string escapeTime = "";
    string escapeGrade = "";

    // mode
    private int mode = 0;
    // Unlocked skills
    private bool canDash = false;
    private bool canWallJump = false;
    // Progress
    private int level = 1;
    private int health = 5;
    private int souls = 0;
    private int kills = 0;
    private int berries = 0;

    private int checkpointLevel = 1;
    private int checkpointKills = 0;
    private int checkpointBerries = 0;

    private int swordDamage = 1;
    private int checkpointSwordDamage = 1;


    [Signal] public delegate void berriesChangedSignal(int value);


    #region SetGets

    int Mode
    {
        set { mode = value; }
        get { return mode; }
    }
    int Level
    {
        set { level = value; }
        get { return level; }
    }
    int Health
    {
        set { health = value; }
        get { return health; }
    }
    int Souls
    {
        set { souls = value; }
        get { return souls; }
    }
    int Kills
    {
        set { kills = value; }
        get { return kills; }
    }
    int Berries
    {
        set { berries = value; }
        get { return berries; }
    }

    bool CanDash
    {
        set { canDash = value; }
        get { return canDash; }
    }
    bool CanWallJump
    {
        set { canWallJump = value; }
        get { return canWallJump; }
    }

    int CheckpointLevel
    {
        set { checkpointLevel = value; }
        get { return checkpointLevel; }
    }
    int CheckpointBerries
    {
        set { checkpointBerries = value; }
        get { return checkpointBerries; }
    }
    int CheckpointKills
    {
        set { checkpointKills = value; }
        get { return checkpointKills; }
    }
    int SwordDamage
    {
        set { swordDamage = value; }
        get { return swordDamage; }
    }
    int CheckpointSwordDamage
    {
        set { checkpointSwordDamage = value; }
        get { return checkpointSwordDamage; }
    }

    #endregion

    public void IncBerries()
    {
        Berries = Berries + 1;
        this.EmitSignal(nameof(berriesChangedSignal), Berries);
    }

    public override void _Ready()
    {
        OS.WindowFullscreen = false;
        UpdateData();
    }

    // update data before saving them
    public void UpdateData()
    {
        _data.Mode = this.Mode;
        _data.Level = this.Level;
        _data.Health = this.Health;
        _data.Souls = this.Souls;
        _data.Berries = this.Berries;
        _data.Kills = this.Kills;

        _data.CanDash = this.canDash;
        _data.CanWallJump = this.canWallJump;

        _data.CheckpointLevel = this.checkpointLevel;
        _data.CheckpointKills = this.checkpointKills;
        _data.CheckpointBerries = this.checkpointBerries;

        _data.SwordDamage = this.swordDamage;
        _data.CheckpointSwordDamage = this.checkpointSwordDamage;
    }

    // update local variables after loading
    public void UpdateVariables()
    {
        this.Mode = _data.Mode;

        this.Level = _data.Level;
        this.Health = _data.Health;
        this.Souls = _data.Souls;
        this.Berries = _data.Berries;
        this.Kills = _data.Kills;

        this.canDash = _data.CanDash;
        this.canWallJump = _data.CanWallJump;

        this.checkpointLevel = _data.CheckpointLevel;
        this.checkpointKills = _data.CheckpointKills;
        this.checkpointBerries = _data.CheckpointBerries;

        this.swordDamage = _data.SwordDamage;
        this.checkpointSwordDamage = _data.CheckpointSwordDamage;
    }

    public void SaveData()
    {
        UpdateData();
        var file = OpenSaveFile(File.ModeFlags.Write);
        if (file != null)
        {
            var json = JsonConvert.SerializeObject(_data);
            file.StoreLine(json);
            file.Close();
        }
    }

    public void LoadData()
    {
        string jsonString = null;
        var file = OpenSaveFile(File.ModeFlags.Read);
        if (file != null)
        {
            jsonString = file.GetLine();
            try
            {
                _data = JsonConvert.DeserializeObject<DataModel>(jsonString);
            }
            catch
            {
                // create new save data if something went wrong
                ResetData();
            }
            file.Close();
            UpdateVariables();
        }
        else
        {
            ResetData();
            UpdateVariables();
        }
    }

    public void ResetData()
    {
        _data = new DataModel();

        _data.Mode = 0;

        _data.Level = 1;
        _data.Health = 5;
        _data.Souls = 0;
        _data.Berries = 0;
        _data.Kills = 0;

        _data.CanDash = false;
        _data.CanWallJump = false;

        _data.CheckpointLevel = 1;
        _data.CheckpointKills = 0;
        _data.CheckpointBerries = 0;

        _data.SwordDamage = 1;
        _data.CheckpointSwordDamage = 1;
    }

    public void DeleteData()
    {
        Directory dir = new Directory();
        dir.Remove(SAVE_PATH);
    }


    private static File OpenSaveFile(File.ModeFlags flag = File.ModeFlags.Read)
    {
        var file = new File();
        if (file.Open(SAVE_PATH, flag) == 0)
        {
            return file;
        }
        return null;
    }

    public override void _PhysicsProcess(float delta)
    {
        if (Input.IsActionJustPressed("key_f"))
        {
            OS.WindowFullscreen = !OS.WindowFullscreen;
        }
    }


}