using Godot;
using System;

public class HealthUI : Control
{
    private int health = 1;
    private int maxHealth = 1;

    Node playerStats;
    Node playerVariables;
    AnimationPlayer animationPlayer;
    TextureRect healthUIFull;
    TextureRect healthUIEmpty;
    Vector2 sizeVector = new Vector2(15, 13); // size of texture in pixels

    Texture hearthAssist;
    Texture hearthChallenge;

    int Health
    {
        set
        {
            health = Mathf.Clamp(value, 0, MaxHealth);
            if (healthUIFull != null)
            {
                sizeVector.x = health * 15;
                healthUIFull.RectSize = sizeVector;
            }
        }
        get { return health; }
    }

    int MaxHealth
    {
        set
        {
            maxHealth = Mathf.Max(1, value);
            Health = Mathf.Min(Health, maxHealth);
            if (healthUIEmpty != null)
            {
                sizeVector.x = maxHealth * 15;
                healthUIEmpty.RectSize = sizeVector;
            }
            playerStats.Set("Health", health);
        }
        get { return maxHealth; }
    }

    public override void _Ready()
    {
        playerStats = GetParent().GetNode<KinematicBody2D>("../Player").GetNode<Node>("Stats");
        playerVariables = GetNode<Node>("/root/PlayerVariables");

        animationPlayer = GetNode<AnimationPlayer>("HealthUIFull/AnimationPlayer");
        healthUIFull = GetNode<TextureRect>("HealthUIFull");
        healthUIEmpty = GetNode<TextureRect>("HealthUIEmpty");

        hearthAssist = (Texture)ResourceLoader.Load("res://UI/health_full_assist.png");
        hearthChallenge = (Texture)ResourceLoader.Load("res://UI/health_full_challenge.png");

        playerStats.Connect("healthChangedSignal", this, "onHealthChangedSignal");
        playerStats.Connect("maxHealthChangedSignal", this, "onMaxHealthChangedSignal");

        if ((int)playerVariables.Get("Mode") == 1)
        {
            healthUIFull.Texture = hearthAssist;
        }
        else if ((int)playerVariables.Get("Mode") == 2)
        {
            healthUIFull.Texture = hearthChallenge;
        }

    }

    public void HealthFlashAnim()
    {
        animationPlayer.Play("Flash");
    }

    public void onHealthChangedSignal(int value)
    {
        Health = value;
    }

    public void onMaxHealthChangedSignal(int value)
    {
        MaxHealth = value;
    }
}
