using Godot;
using System;

public class LastLevelLava : KinematicBody2D
{
    Area2D playerDetection;
    Area2D endArea;
    KinematicBody2D playerBody;
    Tween tween;

    // vectors
    Vector2 motion = new Vector2(0, 0);
    Vector2 upVector = new Vector2(0, -1);
    Vector2 startingPosition = new Vector2(0, 0);

    const int LAVA_SLOW_SPEED = -10;
    const int LAVA_FAST_SPEED = -35;

    int lavaSpeed = 0;
    bool triggered = false;

    public override void _Ready()
    {
        playerBody = GetParent().GetNode<KinematicBody2D>("../Player");
        endArea = GetParent().GetNode<Area2D>("../Elements/EndArea");
        playerDetection = GetParent().GetNode<Area2D>("./LavaTriggerArea");
        tween = GetNode<Tween>("Tween");

        playerDetection.Connect("playerEnteredZoneSignal", this, "TriggerTheLava");
        playerBody.Connect("lastLevelResetSignal", this, "LavaReset");
        endArea.Connect("playerEnteredEndSignal", this, "LavaStop");

        startingPosition = GlobalPosition;
    }

    public override void _PhysicsProcess(float delta)
    {
        motion.y = lavaSpeed;
        motion = MoveAndSlide(motion, upVector);
    }

    public void TriggerTheLava()
    {
        triggered = true;
        lavaSpeed = LAVA_SLOW_SPEED;
    }

    public void LavaReset()
    {
        GlobalPosition = startingPosition;
        lavaSpeed = LAVA_SLOW_SPEED;
    }

    public void LavaStop()
    {
        tween.StopAll();
        tween.InterpolateProperty(this, "lavaSpeed", motion.y, 0, 5f, Tween.TransitionType.Sine, Tween.EaseType.Out);
        tween.Start();
    }

    public void onLavaPlayerNearAreaBodyEntered(KinematicBody2D body)
    {
        if (triggered)
        {
            tween.InterpolateProperty(this, "lavaSpeed", motion.y, LAVA_SLOW_SPEED, 1, Tween.TransitionType.Sine, Tween.EaseType.Out);
            tween.Start();
        }

    }

    public void onLavaPlayerNearAreaBodyExited(KinematicBody2D body)
    {
        if (triggered)
        {
            tween.Stop(this);
            lavaSpeed = LAVA_FAST_SPEED;
        }
    }

}
