using Godot;
using System;

public class LevelSelectButton : Button
{
    Node playerVariables;
    PackedScene audioStreamPlayer;
    AudioStreamSample asFocus;
    AudioStreamSample asSelect;

    [Export]
    string levelPath = "";
    [Export]
    bool startFocused = false;
    [Export]
    int mode = 0;
    [Export]
    bool deleteSave = false;
    [Export]
    bool canDash = false;
    [Export]
    bool canWallJump = false;
    [Export]
    int swordDamage = 1;
    [Export]
    int checkpointLevel = 1;

    public override void _Ready()
    {
        playerVariables = GetNode<Node>("/root/PlayerVariables");
        audioStreamPlayer = (PackedScene)ResourceLoader.Load("res://Sound/AudioStreamPlayer.tscn");
        asFocus = (AudioStreamSample)ResourceLoader.Load("res://Sound/m_button.wav");
        asSelect = (AudioStreamSample)ResourceLoader.Load("res://Sound/m_select.wav");

        Connect("mouse_entered", this, "onButtonMouseEntered");
        Connect("focus_exited", this, "onFocused");
        Connect("button_down", this, "onDown");
        Connect("pressed", this, "onButtonPressed");

        if (startFocused)
        {
            GrabFocus();
        }
    }

    public void PlaySound(AudioStreamSample sample)
    {
        AudioStreamPlayer asp = (AudioStreamPlayer)audioStreamPlayer.Instance();
        asp.Stream = sample;
        GetTree().CurrentScene.CallDeferred("add_child", asp);
    }

    public void onButtonMouseEntered()
    {
        GrabFocus();
    }

    public void onFocused()
    {
        PlaySound(asFocus);
    }

    public void onDown()
    {
        PlaySound(asSelect);
    }

    public void onButtonPressed()
    {
        if (levelPath != "")
        {
            if (deleteSave)
            {
                playerVariables.Call("DeleteData");
                playerVariables.Call("LoadData");
                playerVariables.Set("loaded", true);
                playerVariables.Set("Mode", mode);
                playerVariables.Set("CanDash", canDash);
                playerVariables.Set("CanWallJump", canWallJump);
                playerVariables.Set("SwordDamage", swordDamage);
                playerVariables.Set("CheckpointLevel", checkpointLevel);
                playerVariables.Call("SaveData");
            }
            GetTree().ChangeScene(levelPath);
        }
        else
        {
            GetTree().Quit();
        }
    }


}