using Godot;
using System;
using System.Collections.Generic;

public class MenuContinue : Button
{
    PackedScene audioStreamPlayer;
    AudioStreamSample asFocus;
    AudioStreamSample asSelect;

    [Export]
    bool startFocused = false;

    Node playerVariables;

    private const string SAVE_PATH = @"user://save.json";
    bool canClick = false;

    public override void _Ready()
    {
        audioStreamPlayer = (PackedScene)ResourceLoader.Load("res://Sound/AudioStreamPlayer.tscn");
        asFocus = (AudioStreamSample)ResourceLoader.Load("res://Sound/m_button.wav");
        asSelect = (AudioStreamSample)ResourceLoader.Load("res://Sound/m_select.wav");

        playerVariables = GetNode<Node>("/root/PlayerVariables");

        Connect("mouse_entered", this, "onButtonMouseEntered");
        Connect("focus_exited", this, "onFocused");
        Connect("button_down", this, "onDown");
        Connect("pressed", this, "onButtonPressed");

        if (startFocused)
        {
            GrabFocus();
        }

        var file = new File();
        if (file.FileExists(SAVE_PATH))
        {
            canClick = true;
            setFontColor(new Color(1, 1, 1, 1));
        }
        else
        {
            setFontColor(new Color(0.5f, 0.5f, 0.5f, 1));
        }
    }

    public void PlaySound(AudioStreamSample sample)
    {
        AudioStreamPlayer asp = (AudioStreamPlayer)audioStreamPlayer.Instance();
        asp.Stream = sample;
        GetTree().CurrentScene.CallDeferred("add_child", asp);
    }

    public void setFontColor(Color col)
    {
        Set("custom_colors/font_color", col);
        Set("custom_colors/font_color_hover", col);
        Set("custom_colors/font_color_pressed", col);
    }

    public void onButtonMouseEntered()
    {
        GrabFocus();
    }

    public void onFocused()
    {
        PlaySound(asFocus);
    }

    public void onDown()
    {
        PlaySound(asSelect);
    }

    public void onButtonPressed()
    {
        if (canClick)
        {
            playerVariables.Call("LoadData");
            playerVariables.Set("loaded", true);

            GetTree().ChangeScene("res://Levels/Level_" + playerVariables.Get("Level") + ".tscn");
        }
    }


}
