using Godot;
using System;

public class PlayerCamera : Camera2D
{
    Timer shakeLengthTimer;
    Timer waitTimeTimer;
    Tween tweenShake;
    Sprite hurtSprite;

    float resetSpeed = 0;
    float strength = 0;
    bool doingShake = false;
    int priority = 0;

    public override void _Ready()
    {
        shakeLengthTimer = GetNode<Timer>("ShakeLengthTimer");
        waitTimeTimer = GetNode<Timer>("WaitTimeTimer");
        tweenShake = GetNode<Tween>("TweenCameraShake");
        hurtSprite = GetNode<Sprite>("HurtSprite");
    }

    public void onShakeLengthTimerTimeout()
    {
        doingShake = false;
        priority = 0;
        ResetCamera();
    }

    public void onWaitTimeTimerTimeout()
    {
        if (doingShake)
        {
            Vector2 shakeVector = new Vector2((float)GD.RandRange(-strength, strength), (float)GD.RandRange(-strength, strength));
            tweenShake.InterpolateProperty(this, "offset", Offset, shakeVector, resetSpeed, Tween.TransitionType.Sine, Tween.EaseType.Out);
            tweenShake.Start();
        }
    }

    public void ResetCamera()
    {
        tweenShake.InterpolateProperty(this, "offset", Offset, new Vector2(0, 0), resetSpeed, Tween.TransitionType.Sine, Tween.EaseType.Out);
        tweenShake.Start();
        this.Offset = new Vector2(0, 0);
    }

    public void StartShake(float shakeTime, float shakeSpeed, float shakeStrength, int shakePriority = 1)
    {
        if (doingShake && shakePriority <= priority)
        {
            return;
        }

        doingShake = true;
        priority = shakePriority;
        strength = shakeStrength;
        resetSpeed = shakeSpeed;
        shakeLengthTimer.Start(shakeTime);
        waitTimeTimer.Start(shakeSpeed);
    }

    public void StartFlash(float flashSpeed, float flashStrength)
    {
        tweenShake.InterpolateProperty(hurtSprite, "modulate:a", 0, flashStrength, flashSpeed, Tween.TransitionType.Sine, Tween.EaseType.Out);
        tweenShake.Start();

        tweenShake.InterpolateProperty(hurtSprite, "modulate:a", flashStrength, 0, flashSpeed, Tween.TransitionType.Sine, Tween.EaseType.Out, 0.1f);
        tweenShake.Start();
    }

    public void FadeIn(float duration = 0.5f)
    {
        tweenShake.InterpolateProperty(hurtSprite, "modulate:a", 0, 1, duration, Tween.TransitionType.Sine, Tween.EaseType.Out);
        tweenShake.Start();
    }

    public void FadeOut(float duration = 0.5f)
    {
        tweenShake.InterpolateProperty(hurtSprite, "modulate:a", 1, 0, 0.5f, Tween.TransitionType.Sine, Tween.EaseType.Out);
        tweenShake.Start();
    }

}
