using Godot;
using System;
using System.Collections.Generic;
using Newtonsoft.Json;


public class PlayerGhost : KinematicBody2D
{
    Sprite ghostSprite;
    KinematicBody2D player;
    Sprite playerSprite;
    Node playerVariables;

    private const string GHOST_SAVE_PATH = @"user://ghostdata.json";

    private static GhostDataModel _ghostData = new GhostDataModel();
    Dictionary<int, GhostDataModel> ghostDataDictionary = new Dictionary<int, GhostDataModel>();


    int ghostFrameCount = 0;
    Vector2 ghostPosition = new Vector2(0, 0);


    public override void _Ready()
    {
        ghostSprite = GetNode<Sprite>("Sprite");
        player = GetParent().GetNode<KinematicBody2D>("Player");
        playerSprite = player.GetNode<Sprite>("Sprite");
        playerVariables = GetNode<Node>("/root/PlayerVariables");

        player.Connect("lastLevelResetSignal", this, "GhostReset");

        ghostSprite.Texture = playerSprite.Texture;
        ghostSprite.Hframes = playerSprite.Hframes;
        ghostSprite.Vframes = playerSprite.Vframes;

        if ((int)playerVariables.Get("Mode") == 3)
        {
            LoadGhostFile();
        }
    }

    public override void _PhysicsProcess(float delta)
    {
        if ((int)player.Get("state") != 9 && (int)player.Get("state") != 10)
        {
            GetGhostData();
        }
        else
        {
            ghostSprite.GlobalPosition = new Vector2(0, 0);
        }
    }

    public void GhostReset()
    {
        ghostFrameCount = 0;
    }


    public void LoadGhostFile()
    {
        string jsonString = null;
        var file = OpenSaveFile(File.ModeFlags.Read);
        if (file != null)
        {
            jsonString = file.GetLine();
            try
            {
                ghostDataDictionary = JsonConvert.DeserializeObject<Dictionary<int, GhostDataModel>>(jsonString);
            }
            catch
            {
                DeleteData();
            }
            file.Close();
        }
    }

    public void GetGhostData()
    {
        ghostFrameCount++;
        if (ghostDataDictionary.ContainsKey(ghostFrameCount))
        {
            _ghostData = (GhostDataModel)ghostDataDictionary[ghostFrameCount];

            ghostSprite.GlobalPosition = (new Vector2(_ghostData.PositionX, _ghostData.PositionY));
            ghostSprite.Frame = _ghostData.Frame;
            ghostSprite.FlipH = _ghostData.FlipH;
        }
    }


    public void DeleteData()
    {
        Directory dir = new Directory();
        dir.Remove(GHOST_SAVE_PATH);
    }


    private static File OpenSaveFile(File.ModeFlags flag = File.ModeFlags.Read)
    {
        var file = new File();
        if (file.Open(GHOST_SAVE_PATH, flag) == 0)
        {
            return file;
        }
        return null;
    }

}
