using Godot;
using System;

public class SoulBar : TextureProgress
{
    Node playerStats;
    Node playerVariables;
    AnimationPlayer animationPlayer;
    TextureProgress soulProgress;
    Timer soulBarTimer;
    Tween tween;
    Control hearths;
    PackedScene audioStreamPlayer;
    AudioStreamSample asHpUp;

    private int soulsValue;
    int soulNew;
    float animationDuration = 1f;
    float animationFullDuration;

    int SoulsValue
    {
        set
        {
            SoulFlashAnim();
            AnimateValue(value, animationDuration);
            soulsValue = value;
        }
        get { return soulsValue; }
    }


    public override void _Ready()
    {
        playerStats = GetParent().GetNode<KinematicBody2D>("../Player").GetNode<Node>("Stats");
        playerVariables = GetNode<Node>("/root/PlayerVariables");

        animationPlayer = GetNode<AnimationPlayer>("AnimationPlayer");
        hearths = GetNode<Control>("../HearthUI");
        soulProgress = GetNode<TextureProgress>("SoulProgress");
        tween = GetNode<Tween>("Tween");
        soulBarTimer = GetNode<Timer>("SoulBarTimer");

        audioStreamPlayer = (PackedScene)ResourceLoader.Load("res://Sound/AudioStreamPlayer.tscn");
        asHpUp = (AudioStreamSample)ResourceLoader.Load("res://Sound/health_up.wav");

        playerStats.Connect("soulChangedSignal", this, "onSoulChangedSignal");
        playerStats.Connect("soulFullSignal", this, "onSoulFullSignal");

    }

    public void Initialize(int current)
    {
        soulsValue = current;
        soulProgress.Value = current;
    }

    public void SoulFlashAnim()
    {
        animationPlayer.Play("Flash");
    }

    public void AnimateValue(int target, float duration)
    {
        tween.InterpolateProperty(soulProgress, "value", SoulsValue, target, duration, Tween.TransitionType.Sine, Tween.EaseType.Out);
        tween.Start();
    }

    public void AnimateFull(float duration)
    {
        tween.InterpolateProperty(soulProgress, "value", SoulsValue, 100, duration, Tween.TransitionType.Linear, Tween.EaseType.In);
        tween.Start();
    }

    public void onSoulChangedSignal(int value)
    {
        SoulsValue = value;
    }

    public void onSoulFullSignal(int value)
    {
        SoulFlashAnim();
        animationFullDuration = (100 - value) * 0.008f;
        AnimateFull(animationFullDuration);
        soulNew = value;
        soulBarTimer.Start(animationFullDuration);
    }

    public void onSoulBarTimerTimeout()
    {
        if ((int)playerStats.Get("Health") > 0)
        {
            soulsValue = 0; // reset without animation
            SoulsValue = soulNew; // new value with animation
            playerStats.Set("Health", (int)playerStats.Get("Health") + 1);
            hearths.Call("HealthFlashAnim");
            PlaySound(asHpUp);
        }
    }

    public void PlaySound(AudioStreamSample sample)
    {
        AudioStreamPlayer asp = (AudioStreamPlayer)audioStreamPlayer.Instance();
        asp.Stream = sample;
        GetTree().CurrentScene.AddChild(asp);
    }

}
