using Godot;
using System;
using System.Collections.Generic;
using Newtonsoft.Json;

public class SpeedrunRecording : Node2D
{
    KinematicBody2D player;
    Sprite playerSprite;
    Area2D endArea;
    Node stopwatch;
    PackedScene ghost;

    private const string GHOST_SAVE_PATH = @"user://ghostdata.json";
    int ghostFrameCount = 0;
    //Dictionary<int, object> ghostDictionary = new Dictionary<int, Ghost>();
    Dictionary<int, GhostDataModel> ghostDictionary = new Dictionary<int, GhostDataModel>();


    public override void _Ready()
    {
        ghost = (PackedScene)ResourceLoader.Load("res://Player/PlayerGhost.tscn");
        player = GetNode<KinematicBody2D>("Player");
        playerSprite = player.GetNode<Sprite>("Sprite");
        endArea = GetNode<Area2D>("Elements/EndArea");
        stopwatch = player.GetNode<Node>("Stopwatch");

        player.Connect("lastLevelResetSignal", this, "GhostReset");
        stopwatch.Connect("newRecordSignal", this, "GhostSave");

        OpenGhostFile();
    }

    public override void _PhysicsProcess(float delta)
    {
        if ((int)player.Get("state") != 9)
        {
            DoRecord();
        }
    }




    public void OpenGhostFile()
    {
        var file = new File();

        if (file.FileExists(GHOST_SAVE_PATH))
        {
            KinematicBody2D ghostRun = (KinematicBody2D)ghost.Instance();
            CallDeferred("add_child", ghostRun);
        }
    }

    public void DoRecord()
    {
        ghostFrameCount++;

        //ghostDictionary[ghostFrameCount] = player.GlobalPosition;


        GhostDataModel _ghostData = new GhostDataModel();
        _ghostData.PositionX = playerSprite.GlobalPosition.x;
        _ghostData.PositionY = playerSprite.GlobalPosition.y;
        _ghostData.Frame = playerSprite.Frame;
        _ghostData.FlipH = playerSprite.FlipH;

        ghostDictionary[ghostFrameCount] = _ghostData;

    }

    public void GhostReset()
    {
        ghostDictionary.Clear();
        ghostFrameCount = 0;
    }

    public void GhostSave()
    {
        var file = new File();
        file.Open(GHOST_SAVE_PATH, File.ModeFlags.Write);
        //file.StoreString(JSON.Print(GD.Var2Str(ghostDictionary)));
        //file.Close();

        var json = JsonConvert.SerializeObject(ghostDictionary);
        file.StoreLine(json);
        file.Close();

    }
}
