using Godot;
using System;

public class Stats : Node
{
    Node playerVariables;
    TextureProgress soulBar;

    private int maxHealth = 1;
    private int health = 1;
    private int kills = 0;
    private int berries = 0;
    private int souls = 0;

    [Signal] public delegate void noHealthSignal();
    [Signal] public delegate void healthChangedSignal(int value);
    [Signal] public delegate void maxHealthChangedSignal(int value);

    [Signal] public delegate void soulChangedSignal(int value);
    [Signal] public delegate void soulFullSignal(int value);


    public override void _Ready()
    {
        playerVariables = GetNode<Node>("/root/PlayerVariables");
        soulBar = GetParent().GetNode<TextureProgress>("../UI/SoulBar");

        // start of level initialization
        MaxHealth = 5;
        Health = (int)playerVariables.Get("Health");
        Kills = (int)playerVariables.Get("Kills");
        Berries = (int)playerVariables.Get("Berries");

        // initialize without animation
        souls = (int)playerVariables.Get("Souls");
        soulBar.Call("Initialize", souls);

    }

    int Health
    {
        set
        {
            if (Souls == 100)
            {
                if ((int)playerVariables.Get("Mode") != 1)
                {
                    souls = 0;
                    this.EmitSignal(nameof(soulFullSignal), souls);
                }
            }
            else
            {
                health = Mathf.Min(value, MaxHealth);
                this.EmitSignal(nameof(healthChangedSignal), health);
                if (health <= 0)
                {
                    this.EmitSignal(nameof(noHealthSignal));
                }
            }

        }
        get { return health; }
    }

    int MaxHealth
    {
        set
        {
            maxHealth = value;
            Health = Mathf.Min(health, maxHealth);
            this.EmitSignal(nameof(maxHealthChangedSignal), maxHealth);
        }
        get { return maxHealth; }
    }


    int Souls
    {
        set
        {
            souls = value;
            if (souls < 100)
            {
                this.EmitSignal(nameof(soulChangedSignal), souls);
            }
            else if (Health == MaxHealth)
            {
                souls = 100;
                this.EmitSignal(nameof(soulChangedSignal), souls);
            }
            else
            {
                souls -= 100;
                this.EmitSignal(nameof(soulFullSignal), souls);
            }
        }
        get { return souls; }
    }

    int Kills
    {
        set
        {
            kills = value;
        }
        get
        {
            return kills;
        }
    }

    int Berries
    {
        set
        {
            berries = value;
        }
        get
        {
            return berries;
        }
    }

}
