using Godot;
using System;
using Newtonsoft.Json;

public class Stopwatch : Node
{
    private const string TIME_SAVE_PATH = @"user://toptime.json";

    Node playerVariables;
    KinematicBody2D player;
    Area2D endArea;
    Area2D lavaTriggerArea;
    Timer msTimer;
    RichTextLabel text1;
    RichTextLabel text2;
    RichTextLabel text3;
    RichTextLabel text4;

    bool counting = false;

    int ms = 0;
    int sec = 0;
    int min = 0;

    int timeOfStart = 0;
    int timePassed = 0;
    int bestEscapeTime = 0;

    string timeString;
    string escapeGrade;
    string msZero = "";
    string secZero = "";

    [Signal] public delegate void newRecordSignal();

    public override void _Ready()
    {
        playerVariables = GetNode<Node>("/root/PlayerVariables");
        player = (KinematicBody2D)GetParent();
        endArea = GetParent().GetNode<Area2D>("../Elements/EndArea");
        lavaTriggerArea = GetParent().GetNode<Area2D>("../Lava/LavaTriggerArea");
        msTimer = GetNode<Timer>("msTimer");
        text1 = GetNode<RichTextLabel>("TextOnScreen1");
        text2 = GetNode<RichTextLabel>("TextOnScreen2");
        text3 = GetNode<RichTextLabel>("TextOnScreen3");
        text4 = GetNode<RichTextLabel>("TextOnScreen4");

        lavaTriggerArea.Connect("playerEnteredZoneSignal", this, "StartCounting");
        player.Connect("fromPreparedState", this, "StartCounting");
        player.Connect("lastLevelResetSignal", this, "ResetCounting");
        endArea.Connect("playerEnteredEndSignal", this, "SaveTime");


        if ((int)playerVariables.Get("Mode") != 3)
        {
            text1.Visible = false;
            text2.Visible = false;
            text3.Visible = false;
            text4.Visible = false;
        }

        LoadTimeData();

    }


    public override void _Process(float delta)
    {
        if (counting)
        {
            timePassed = (int)OS.GetTicksMsec() - timeOfStart;
            ms = timePassed % 1000;
            sec = ((timePassed - ms) / 1000);
            min = (int)(sec / 60);
            sec = sec % 60;
            ms = ms / 10;


            if (sec < 10) secZero = "0";
            else secZero = "";
            if (ms < 10) msZero = "0";
            else msZero = "";

            timeString = min + ":" + secZero + sec + "." + msZero + ms;
            text1.Text = timeString;
            text2.Text = timeString;
            text3.Text = timeString;
            text4.Text = timeString;
        }
    }

    public void StartCounting()
    {
        if ((int)player.Get("state") != 9)
        {
            counting = true;
            timeOfStart = (int)OS.GetTicksMsec();
        }
    }

    public void ResetCounting()
    {
        counting = false;
        text1.Text = "0:00.00";
        text2.Text = "0:00.00";
        text3.Text = "0:00.00";
        text4.Text = "0:00.00";
    }

    public void SaveTime()
    {
        counting = false;
        playerVariables.Set("escapeTime", timeString);
        playerVariables.Set("BestEscapeTime", timePassed);


        if (timePassed < bestEscapeTime)
        {
            this.EmitSignal(nameof(newRecordSignal));
            SaveTimeData();
        }


        if (timePassed < 35000) escapeGrade = "GODLIKE";
        else if (timePassed < 40000) escapeGrade = "A+";
        else if (timePassed < 45000) escapeGrade = "A";
        else if (timePassed < 50000) escapeGrade = "B+";
        else if (timePassed < 55000) escapeGrade = "B";
        else if (timePassed < 60000) escapeGrade = "C+";
        else if (timePassed < 65000) escapeGrade = "C";
        else if (timePassed < 70000) escapeGrade = "D+";
        else if (timePassed < 75000) escapeGrade = "D";
        else if (timePassed < 80000) escapeGrade = "E+";
        else if (timePassed < 85000) escapeGrade = "E";
        else escapeGrade = "F";

        playerVariables.Set("escapeGrade", escapeGrade);
    }


    public void SaveTimeData()
    {
        var file = new File();
        file.Open(TIME_SAVE_PATH, File.ModeFlags.Write);
        file.StoreString(JSON.Print(timePassed));
        file.Close();
    }

    public void LoadTimeData()
    {
        string jsonString = null;
        var file = OpenSaveFile(File.ModeFlags.Read);
        if (file != null)
        {
            jsonString = file.GetLine();
            try
            {
                bestEscapeTime = JsonConvert.DeserializeObject<int>(jsonString);
            }
            catch
            {
                DeleteData();
            }
            file.Close();
        }
        else
        {
            bestEscapeTime = Int32.MaxValue;
        }
    }

    private static File OpenSaveFile(File.ModeFlags flag = File.ModeFlags.Read)
    {
        var file = new File();
        if (file.Open(TIME_SAVE_PATH, flag) == 0)
        {
            return file;
        }
        return null;
    }


    public void DeleteData()
    {
        Directory dir = new Directory();
        dir.Remove(TIME_SAVE_PATH);
    }



}
